# -*- coding: utf-8 -*-

# MIT license
#
# Copyright (C) 2018 by XESS Corp.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""
Definitions used everywhere.
"""

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import

# Places where parts can be stored.
#   NETLIST: The part will become part of a circuit netlist.
#   LIBRARY: The part will be placed in the part list for a library.
#   TEMPLATE: The part will be used as a template to be copied from.
from future import standard_library
standard_library.install_aliases()
NETLIST, LIBRARY, TEMPLATE = ['NETLIST', 'LIBRARY', 'TEMPLATE']

# Prefixes for implicit nets and buses.
NET_PREFIX = 'N$'
BUS_PREFIX = 'B$'

# Supported ECAD tools.
ALL_TOOLS = KICAD, SKIDL, SPICE = ['kicad', 'skidl', 'spice']

# Utility for changing the net and bus prefixes.
def set_net_bus_prefixes(net, bus):
    global NET_PREFIX
    global BUS_PREFIX
    NET_PREFIX = net
    BUS_PREFIX = bus



# -*- coding: utf-8 -*-

# MIT license
#
# Copyright (C) 2016 by XESS Corp.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
"""
Utility functions.
"""

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import
from builtins import super
from builtins import open
from builtins import int
from builtins import dict
from builtins import str
from builtins import zip
from builtins import range
from builtins import object
from future import standard_library
standard_library.install_aliases()

import sys
import os
import os.path
import re
import logging
import inspect
import traceback
from contextlib import contextmanager

from .py_2_3 import *

# Separator for strings containing multiple indices.
INDEX_SEPARATOR = ','


def norecurse(f):
    """This decorator will keep function f from recursively calling itself."""

    def func(*args, **kwargs):
        # If a function's name is on the stack twice (once for the current call
        # and a second time for the previous call), then return without
        # executing the function.
        if len([1 for l in traceback.extract_stack()
                if l[2] == f.__name__]) > 1:
            return None

        # Otherwise, not a recursive call so execute the function and return result.
        return f(*args, **kwargs)

    return func


class CountCalls(object):
    """
    Decorator for counting the number of times a function is called.

    This is used for counting errors and warnings passed to logging functions,
    making it easy to track if and how many errors/warnings were issued.
    """

    def __init__(self, func):
        self.func = func
        self.count = 0

    def __call__(self, *args, **kwargs):
        self.count += 1
        return self.func(*args, **kwargs)


def scriptinfo():
    """
    Returns a dictionary with information about the running top level Python
    script:
    ---------------------------------------------------------------------------
    dir:    directory containing script or compiled executable
    name:   name of script or executable
    source: name of source code file
    ---------------------------------------------------------------------------
    "name" and "source" are identical if and only if running interpreted code.
    When running code compiled by py2exe or cx_freeze, "source" contains
    the name of the originating Python script.
    If compiled by PyInstaller, "source" contains no meaningful information.

    Downloaded from:
    http://code.activestate.com/recipes/579018-python-determine-name-and-directory-of-the-top-lev/
    """

    #---------------------------------------------------------------------------
    # scan through call stack for caller information
    #---------------------------------------------------------------------------
    trc = 'skidl'  # Make sure this gets set to something when in interactive mode.
    for teil in inspect.stack():
        # skip system calls
        if teil[1].startswith("<"):
            continue
        if teil[1].upper().startswith(sys.exec_prefix.upper()):
            continue
        trc = teil[1]

    # trc contains highest level calling script name
    # check if we have been compiled
    if getattr(sys, 'frozen', False):
        scriptdir, scriptname = os.path.split(sys.executable)
        return {"dir": scriptdir, "name": scriptname, "source": trc}

    # from here on, we are in the interpreted case
    scriptdir, trc = os.path.split(trc)
    # if trc did not contain directory information,
    # the current working directory is what we need
    if not scriptdir:
        scriptdir = os.getcwd()

    scr_dict = {"name": trc, "source": trc, "dir": scriptdir}
    return scr_dict


def get_script_name():
    """Return the name of the top-level script."""
    return os.path.splitext(scriptinfo()['name'])[0]


def create_logger(title, log_msg_id='', log_file_suffix='.log'):
    """
    Create a logger, usually for run-time errors or ERC violations.
    """

    logger = logging.getLogger(title)

    # Errors & warnings always appear on the terminal.
    handler = logging.StreamHandler(sys.stderr)
    handler.setLevel(logging.WARNING)
    handler.setFormatter(
        logging.Formatter(log_msg_id + '%(levelname)s: %(message)s'))
    logger.addHandler(handler)

    # Errors and warnings are stored in a log file with the top-level script's name.
    handler = logging.StreamHandler(
        open(get_script_name() + log_file_suffix, 'w'))
    handler.setLevel(logging.WARNING)
    handler.setFormatter(
        logging.Formatter(log_msg_id + '%(levelname)s: %(message)s'))
    logger.addHandler(handler)

    # Set logger to trigger on info, warning, and error messages.
    logger.setLevel(logging.INFO)

    # Augment the logger's functions to count the number of errors and warnings.
    logger.error = CountCalls(logger.error)
    logger.warning = CountCalls(logger.warning)

    return logger


###############################################################################
# Set up loggers for runtime messages and ERC reports.

logger = create_logger('skidl')
erc_logger = create_logger('ERC_Logger', 'ERC ', '.erc')

###############################################################################


def is_binary_file(filename):
    '''Return true if a file contains binary (non-text) characters.'''
    text_chars = bytearray({7, 8, 9, 10, 12, 13, 27}
                           | set(range(0x20, 0x100)) - {0x7f})
    try:
        with open(filename, 'rb') as fp:
            return bool(fp.read(1024).translate(None, text_chars))
    except (IOError, FileNotFoundError, TypeError):
        return False


def find_and_open_file(filename,
                       paths=None,
                       ext=None,
                       allow_failure=False,
                       exclude_binary=False,
                       descend=0):
    """
    Search for a file in list of paths, open it and return file pointer and full file name.

    Args:
        filename: Base file name (e.g., "my_file").
        paths: List of paths to search for the file.
        ext: The extension for the file (e.g., "txt").
        allow_failure: If false, failure to find file raises and exception.
        exclude_binary: If true, skip files that contain binary data.
        descend: If 0, don't search lower-level directories. If positive, search
                 that many levels down for the file. If negative, descend into
                 subdirectories without limit.
    """

    # If no paths are given, then just check the current directory.
    if not paths:
        paths = ['.']

    # If the filename has no extension, then give it one.
    if ext and not filename.endswith(ext):
        fn_plus_ext = filename + ext
    else:
        fn_plus_ext = filename

    # Search the paths for the file.
    for path in paths:
        if not os.path.exists(path):
            continue  # Skip paths that don't exist.
        abs_filename = os.path.join(path, fn_plus_ext)
        if not exclude_binary or not is_binary_file(abs_filename):
            try:
                # The search stops once the file is successfully opened.
                return open(abs_filename), abs_filename
            except (IOError, FileNotFoundError, TypeError):
                pass
        # If file not found, look in subdirectories or go to next path.
        if descend:
            # Look in subdirectories.
            dir_contents = [os.path.join(path, f) for f in os.listdir(path)]
            subdirs = [f for f in dir_contents if os.path.isdir(f)]
            if subdirs:
                fp, fn = find_and_open_file(
                    filename=filename,
                    paths=subdirs,
                    ext=ext,
                    allow_failure=True,
                    exclude_binary=exclude_binary,
                    descend=descend - 1)
                if fp:
                    return fp, fn

    # Couldn't find the file.
    if allow_failure:
        return None, None
    else:
        logger.error("Can't open file: {}.\n".format(filename))
        raise FileNotFoundError


def add_quotes(s):
    """Return string with added quotes if it contains whitespace or parens."""
    if not isinstance(s, basestring):
        return s
    if re.search(r'[\s()]', s):
        return '"' + s + '"'
    return s


def to_list(x):
    """
    Return x if it is already a list, or return a list if x is a scalar.
    """
    if isinstance(x, (list, tuple)):
        return x  # Already a list, so just return it.
    return [x]  # Wasn't a list, so make it into one.


def cnvt_to_var_name(s):
    """Convert a string to a legal Python variable name and return it."""
    return re.sub(r'\W|^(?=\d)', '_', s)


def list_or_scalar(lst):
    """
    Return a list if passed a multi-element list, otherwise return a single scalar.

    Args:
        lst: Either a list or a scalar.

    Returns:
        * A list if passed a multi-element list.
        * The list element if passed a single-element list.
        * None if passed an empty list.
        * A scalar if passed a scalar.
    """
    if isinstance(lst, (list, tuple)):
        if len(lst) > 1:
            return lst  # Multi-element list, so return it unchanged.
        if len(lst) == 1:
            return lst[0]  # Single-element list, so return the only element.
        return None  # Empty list, so return None.
    return lst  # Must have been a scalar, so return that.


def flatten(nested_list):
    """
    Return a flattened list of items from a nested list.
    """
    lst = []
    for item in nested_list:
        if isinstance(item, (list, tuple)):
            lst.extend(flatten(item))
        else:
            lst.append(item)
    return lst


def get_unique_name(lst, attrib, prefix, initial=None):
    """
    Return a name that doesn't collide with another in a list.

    This subroutine is used to generate unique part references (e.g., "R12")
    or unique net names (e.g., "N$5").

    Args:
        lst: The list of objects containing names.
        attrib: The attribute in each object containing the name.
        prefix: The prefix attached to each name.
        initial: The initial setting of the name (can be None or empty string).

    Returns:
        A string containing the unique name.
    """

    # If the initial name is None, then create a name based on the prefix
    # and the smallest unused number that's available for that prefix.
    if not initial:

        # Get list entries with the prefix followed by a number, e.g.: C55
        filter_dict = {attrib: re.escape(prefix) + r'\d+'}
        sub_list = filter_list(lst, **filter_dict)

        # If entries were found, then find the smallest available number.
        if sub_list:
            # Get the list of names.
            names = [getattr(item, attrib) for item in sub_list]
            # Remove the prefix from each name, leaving only the numbers.
            l = len(prefix)
            nums = set([int(n[l:]) for n in names])
            stop = max(nums) + 1
            # Generate a list of the unused numbers in the range [1,stop]
            # and select the minimum value.
            n = min(set(range(1, stop + 1)) - nums)

        # If no entries were found, start counting from 1.
        else:
            n = 1

        # The initial name is the prefix plus the number.
        initial = prefix + str(n)

    # If the initial name is just a number, then prepend the prefix to it.
    elif isinstance(initial, int):
        initial = prefix + str(initial)

    # Now determine if there are any items in the list with the same name.
    filter_dict = {attrib: re.escape(initial)}
    sub_list = filter_list(lst, **filter_dict)

    # If the name is unique, then return it.
    if not sub_list:
        return initial

    # Otherwise, determine how many copies of the name are in the list and
    # append a number to make this name unique.
    filter_dict = {attrib: re.escape(initial) + r'_\d+'}
    n = len(filter_list(lst, **filter_dict))
    initial = initial + '_' + str(n + 1)

    # Recursively call this routine using the newly-generated name to
    # make sure it's unique. Eventually, a unique name will be returned.
    return get_unique_name(lst, attrib, prefix, initial)


def fullmatch(regex, string, flags=0):
    """Emulate python-3.4 re.fullmatch()."""
    return re.match("(?:" + regex + r")\Z", string, flags=flags)


def filter_list(lst, **criteria):
    """
    Return a list of objects whose attributes match a set of criteria.

    Return a list of objects extracted from a list whose attributes match a
    set of criteria. The match is done using regular expressions.
    Example: filter_list(pins, name='io[0-9]+', direction='bidir') will
    return all the bidirectional pins of the component that have pin names
    starting with 'io' followed by a number (e.g., 'IO45').

    If an attribute of the lst object is a list or tuple, each entry in the
    list/tuple will be checked for a match. Only one entry needs to match to
    consider the entire attribute a match. This feature is useful when
    searching for objects that contain a list of aliases, such as Part objects.

    Args:
        lst: The list from which objects will be extracted.

    Keywords Args:
        criteria: Keyword-argument pairs. The keyword specifies the attribute
            name while the argument contains the desired value of the attribute.
            Regardless of what type the argument is, it is always compared as if
            it was a string. The argument can also be a regular expression that
            must match the entire string created from the attribute of the list
            object.

    Returns:
        A list of objects whose attributes match *all* the criteria.
    """

    # Place any matching objects from the list in here.
    extract = []

    for item in lst:
        # Compare an item's attributes to each of the criteria.
        # Break out of the criteria loop and don't add the item to the extract
        # list if *any* of the item's attributes *does not* match.
        for k, v in criteria.items():

            try:
                attr_val = getattr(item, k)
            except AttributeError:
                # If the attribute doesn't exist, then that's a non-match.
                break

            if isinstance(v, (int, basestring)):
                # Check integer or string attributes.

                if isinstance(attr_val, (list, tuple)):
                    # If the attribute value from the item is a list or tuple,
                    # loop through the list of attribute values. If at least one
                    # value matches the current criterium, then break from the
                    # criteria loop and extract this item.
                    for val in attr_val:
                        if fullmatch(str(v), str(val), flags=re.IGNORECASE):
                            # One of the list of values matched, so break from this
                            # loop and do not execute the break in the
                            # loop's else clause.
                            break
                    else:
                        # If we got here, then none of the values in the attribute
                        # list matched the current criterium. Therefore, break out
                        # of the criteria loop and don't add this list item to
                        # the extract list.
                        break
                else:
                    # If the attribute value from the item in the list is a scalar,
                    # see if the value matches the current criterium. If it doesn't,
                    # then break from the criteria loop and don't extract this item.
                    if not fullmatch(
                            str(v), str(attr_val), flags=re.IGNORECASE):
                        break

            else:
                # Check non-integer, non-string attributes.
                if isinstance(attr_val, (list, tuple)):
                    if v not in attr_val:
                        break
                elif v != attr_val:
                    break

        else:
            # If we get here, then all the item attributes matched and the
            # for criteria loop didn't break, so add this item to the
            # extract list.
            extract.append(item)

    return extract


def expand_indices(slice_min, slice_max, *indices):
    """
    Expand a list of indices into a list of integers and strings.

    This function takes the indices used to select pins of parts and 
    lines of buses and returns a flat list of numbers and strings.
    String and integer indices are put in the list unchanged, but
    slices are expanded into a list of integers before entering the
    final list.

    Args:
        slice_min: The minimum possible index.
        slice_max: The maximum possible index (used for slice indices).
        indices: A list of indices made up of numbers, slices, text strings.
            The list can also be nested.

    Returns:
        A linear list of all the indices made up only of numbers and strings.
    """

    def expand_slice(slc):
        """Expand slice notation."""

        # Get bounds for slice.
        start, stop, step = slc.indices(slice_max)
        start = min(max(start, slice_min), slice_max)
        stop = min(max(stop, slice_min), slice_max)

        # Do this if it's a downward slice (e.g., [7:0]).
        if start > stop:
            if slc.start and slc.start > slice_max:
                logger.error('Index out of range ({} > {})!'.format(
                    slc.start, slice_max))
                raise Exception
            # Count down from start to stop.
            stop = stop - step
            step = -step

        # Do this if it's a normal (i.e., upward) slice (e.g., [0:7]).
        else:
            if slc.stop and slc.stop > slice_max:
                logger.error('Index out of range ({} > {})!'.format(
                    slc.stop, slice_max))
                raise Exception
            # Count up from start to stop
            stop += step

        # Create the sequence of indices.
        return range(start, stop, step)

    # Expand each index and add it to the list.
    ids = []
    for indx in flatten(indices):
        if isinstance(indx, slice):
            ids.extend(expand_slice(indx))
        elif isinstance(indx, int):
            ids.append(indx)
        elif isinstance(indx, basestring):
            # String might contain multiple indices with a separator.
            for id in indx.split(INDEX_SEPARATOR):
                ids.append(id.strip())
        else:
            logger.error('Unknown type in index: {}.'.format(type(indx)))
            raise Exception

    # Return the completely expanded list of indices.
    return ids


def find_num_copies(**attribs):
    """
    Return the number of copies to based on the number of attribute values.

    Keyword Args:
        attribs: Dict of Keyword/Value pairs for setting object attributes.
            If the value is a scalar, then the number of copies is one.
            If the value is a list/tuple, the number of copies is the
            length of the list/tuple.

    Returns:
        The length of the longest value in the dict of attributes.

    Raises:
        Exception if there are two or more list/tuple values with different
        lengths that are greater than 1. (All attribute values must be scalars
        or lists/tuples of the same length.)
    """
    num_copies = set()
    for k, v in attribs.items():
        if isinstance(v, (list, tuple)):
            num_copies.add(len(v))
        else:
            num_copies.add(1)

    num_copies = list(num_copies)
    if len(num_copies) > 2:
        logger.error(
            "Mismatched lengths of attributes: {}!".format(num_copies))
        raise Exception
    elif len(num_copies) > 1 and min(num_copies) > 1:
        logger.error(
            "Mismatched lengths of attributes: {}!".format(num_copies))
        raise Exception

    try:
        return max(num_copies)
    except ValueError:
        return 0  # If the list if empty.


@contextmanager
def opened(f_or_fn, mode):
    """
    Yields an opened file or file-like object.

    Args:
       file_or_filename: Either an already opened file or file-like
           object, or a filename to open.
       mode: The mode to open the file in.
    """
    if isinstance(f_or_fn, basestring):
        with open(f_or_fn, mode) as f:
            yield f
    elif hasattr(f_or_fn, "fileno"):
        if mode.replace("+", "") == f_or_fn.mode.replace("+", ""):
            # same mode, can reuse file handle
            yield f_or_fn
        else:
            # open in new mode
            with os.fdopen(f_or_fn.fileno(), mode) as f:
                yield f
    else:
        raise TypeError(
            "argument must be a filename or a file-like object (is: {})".
            format(type(f_or_fn)))


def expand_buses(pins_nets_buses):
    """
    Take list of pins, nets, and buses and return a list of only pins and nets.
    """

    from .Bus import Bus

    pins_nets = []
    for pnb in pins_nets_buses:
        if isinstance(pnb, Bus):
            pins_nets.extend(pnb.get_nets())
        else:
            pins_nets.append(pnb)
    return pins_nets



from skidl import Pin, Part, SchLib, SKIDL, TEMPLATE

SKIDL_lib_version = '0.0.1'

dsp = SchLib(tool=SKIDL).add_parts(*[
        Part(name='DSP96002',dest=TEMPLATE,tool=SKIDL,keywords='DSP 32bit Dual Port Processor',description='32-bit General Purpose Floating-point DSP, Dual Port, PGA-223',ref_prefix='U',num_units=1,fplist=['PGA-223*'],do_erc=True,pins=[
            Pin(num='A1',name='B.A23',func=Pin.TRISTATE,do_erc=True),
            Pin(num='B1',name='B.A20',func=Pin.TRISTATE,do_erc=True),
            Pin(num='C1',name='B.A17',func=Pin.TRISTATE,do_erc=True),
            Pin(num='D1',name='B.A15',func=Pin.TRISTATE,do_erc=True),
            Pin(num='E1',name='B.A13',func=Pin.TRISTATE,do_erc=True),
            Pin(num='F1',name='B.A12',func=Pin.TRISTATE,do_erc=True),
            Pin(num='G1',name='B.A9',func=Pin.TRISTATE,do_erc=True),
            Pin(num='H1',name='B.A8',func=Pin.TRISTATE,do_erc=True),
            Pin(num='J1',name='A.TACK',do_erc=True),
            Pin(num='K1',name='B.A4',func=Pin.TRISTATE,do_erc=True),
            Pin(num='L1',name='B.A3',func=Pin.TRISTATE,do_erc=True),
            Pin(num='M1',name='B.A0',func=Pin.TRISTATE,do_erc=True),
            Pin(num='N1',name='B.AENB',do_erc=True),
            Pin(num='P1',name='B.R/W',func=Pin.TRISTATE,do_erc=True),
            Pin(num='R1',name='B.BUS_STR',func=Pin.OUTPUT,do_erc=True),
            Pin(num='T1',name='B.BG',do_erc=True),
            Pin(num='U1',name='B.HOSTREQ',func=Pin.OUTPUT,do_erc=True),
            Pin(num='V1',name='DSO',func=Pin.OUTPUT,do_erc=True),
            Pin(num='A2',name='B.A27',func=Pin.TRISTATE,do_erc=True),
            Pin(num='B2',name='B.A25',func=Pin.TRISTATE,do_erc=True),
            Pin(num='C2',name='B.A21',func=Pin.TRISTATE,do_erc=True),
            Pin(num='D2',name='B.A18',func=Pin.TRISTATE,do_erc=True),
            Pin(num='E2',name='B.A16',func=Pin.TRISTATE,do_erc=True),
            Pin(num='F2',name='B.A14',func=Pin.TRISTATE,do_erc=True),
            Pin(num='G2',name='B.A10',func=Pin.TRISTATE,do_erc=True),
            Pin(num='H2',name='CLK',do_erc=True),
            Pin(num='J2',name='B.TACK',do_erc=True),
            Pin(num='K2',name='B.A5',func=Pin.TRISTATE,do_erc=True),
            Pin(num='L2',name='B.A1',func=Pin.TRISTATE,do_erc=True),
            Pin(num='M2',name='B.S1',func=Pin.TRISTATE,do_erc=True),
            Pin(num='P2',name='B.T_STROBE',func=Pin.TRISTATE,do_erc=True),
            Pin(num='R2',name='B.BR',func=Pin.OUTPUT,do_erc=True),
            Pin(num='T2',name='B.BA',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='U2',name='DSCK/OS1',func=Pin.BIDIR,do_erc=True),
            Pin(num='V2',name='DSI/OS0',func=Pin.BIDIR,do_erc=True),
            Pin(num='A3',name='B.A29',func=Pin.TRISTATE,do_erc=True),
            Pin(num='B3',name='B.A28',func=Pin.TRISTATE,do_erc=True),
            Pin(num='C3',name='B.A26',func=Pin.TRISTATE,do_erc=True),
            Pin(num='D3',name='B.A24',func=Pin.TRISTATE,do_erc=True),
            Pin(num='E3',name='B.A22',func=Pin.TRISTATE,do_erc=True),
            Pin(num='F3',name='B.A19',func=Pin.TRISTATE,do_erc=True),
            Pin(num='G3',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='H3',name='B.A11',func=Pin.TRISTATE,do_erc=True),
            Pin(num='J3',name='B.A7',func=Pin.TRISTATE,do_erc=True),
            Pin(num='K3',name='B.A6',func=Pin.TRISTATE,do_erc=True),
            Pin(num='L3',name='B.A2',func=Pin.TRISTATE,do_erc=True),
            Pin(num='M3',name='B.S0',func=Pin.TRISTATE,do_erc=True),
            Pin(num='P3',name='B.BUS_LOCK',func=Pin.OUTPUT,do_erc=True),
            Pin(num='R3',name='B.BB',do_erc=True),
            Pin(num='T3',name='A.HOSTREQ',func=Pin.OUTPUT,do_erc=True),
            Pin(num='V3',name='B.HOSTACK',do_erc=True),
            Pin(num='A4',name='B.A31',func=Pin.TRISTATE,do_erc=True),
            Pin(num='B4',name='B.A30',func=Pin.TRISTATE,do_erc=True),
            Pin(num='C4',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='E4',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='F4',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='G4',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='H4',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='J4',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='K4',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='L4',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='M4',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='N4',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='P4',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='R4',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='T4',name='DEBUGREQ',do_erc=True),
            Pin(num='U4',name='A.HOSTACK',do_erc=True),
            Pin(num='V4',name='B.HOSTSEL',do_erc=True),
            Pin(num='A5',name='MODA/IRQA',do_erc=True),
            Pin(num='B5',name='MODB/IRQB',do_erc=True),
            Pin(num='C5',name='MODC/IRQC',do_erc=True),
            Pin(num='D5',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='R5',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='T5',name='A.HOSTSEL',do_erc=True),
            Pin(num='U5',name='B.DENB',do_erc=True),
            Pin(num='V5',name='B.D30',func=Pin.TRISTATE,do_erc=True),
            Pin(num='A6',name='A.BB',do_erc=True),
            Pin(num='B6',name='A.BG',do_erc=True),
            Pin(num='C6',name='RESET',do_erc=True),
            Pin(num='D6',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='R6',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='T6',name='B.D31',func=Pin.TRISTATE,do_erc=True),
            Pin(num='U6',name='B.D29',func=Pin.TRISTATE,do_erc=True),
            Pin(num='V6',name='B.D28',func=Pin.TRISTATE,do_erc=True),
            Pin(num='A7',name='A.BR',func=Pin.OUTPUT,do_erc=True),
            Pin(num='B7',name='A.BA',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='C7',name='A.BUS_LOCK',func=Pin.OUTPUT,do_erc=True),
            Pin(num='D7',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='R7',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='T7',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='U7',name='B.D27',func=Pin.TRISTATE,do_erc=True),
            Pin(num='V7',name='B.D25',func=Pin.TRISTATE,do_erc=True),
            Pin(num='B8',name='B.T_TYPE',func=Pin.OUTPUT,do_erc=True),
            Pin(num='C8',name='A.T_TYPE',func=Pin.OUTPUT,do_erc=True),
            Pin(num='D8',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='R8',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='T8',name='B.D26',func=Pin.TRISTATE,do_erc=True),
            Pin(num='U8',name='B.D24',func=Pin.TRISTATE,do_erc=True),
            Pin(num='V8',name='B.D23',func=Pin.TRISTATE,do_erc=True),
            Pin(num='A9',name='A.R/W',func=Pin.TRISTATE,do_erc=True),
            Pin(num='B9',name='A.S1',func=Pin.TRISTATE,do_erc=True),
            Pin(num='D9',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='R9',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='T9',name='B.D22',func=Pin.TRISTATE,do_erc=True),
            Pin(num='U9',name='B.D21',func=Pin.TRISTATE,do_erc=True),
            Pin(num='V9',name='B.D20',func=Pin.TRISTATE,do_erc=True),
            Pin(num='A10',name='A.S0',func=Pin.TRISTATE,do_erc=True),
            Pin(num='B10',name='A.BUS_STR',func=Pin.OUTPUT,do_erc=True),
            Pin(num='C10',name='A.A1',func=Pin.TRISTATE,do_erc=True),
            Pin(num='D10',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='R10',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='T10',name='B.D17',func=Pin.TRISTATE,do_erc=True),
            Pin(num='U10',name='B.D18',func=Pin.TRISTATE,do_erc=True),
            Pin(num='V10',name='B.D19',func=Pin.TRISTATE,do_erc=True),
            Pin(num='A11',name='A.T_STROBE',func=Pin.TRISTATE,do_erc=True),
            Pin(num='B11',name='A.A0',func=Pin.TRISTATE,do_erc=True),
            Pin(num='C11',name='A.A5',func=Pin.TRISTATE,do_erc=True),
            Pin(num='D11',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='R11',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='T11',name='B.D14',func=Pin.TRISTATE,do_erc=True),
            Pin(num='U11',name='B.D15',func=Pin.TRISTATE,do_erc=True),
            Pin(num='V11',name='B.D16',func=Pin.TRISTATE,do_erc=True),
            Pin(num='A12',name='A.AENB',do_erc=True),
            Pin(num='B12',name='A.A3',func=Pin.TRISTATE,do_erc=True),
            Pin(num='C12',name='A.A8',func=Pin.TRISTATE,do_erc=True),
            Pin(num='D12',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='R12',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='T12',name='B.D11',func=Pin.TRISTATE,do_erc=True),
            Pin(num='U12',name='B.D12',func=Pin.TRISTATE,do_erc=True),
            Pin(num='V12',name='B.D13',func=Pin.TRISTATE,do_erc=True),
            Pin(num='A13',name='A.A2',func=Pin.TRISTATE,do_erc=True),
            Pin(num='B13',name='A.A6',func=Pin.TRISTATE,do_erc=True),
            Pin(num='C13',name='A.A12',func=Pin.TRISTATE,do_erc=True),
            Pin(num='D13',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='R13',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='T13',name='B.D7',func=Pin.TRISTATE,do_erc=True),
            Pin(num='U13',name='B.D9',func=Pin.TRISTATE,do_erc=True),
            Pin(num='V13',name='B.D10',func=Pin.TRISTATE,do_erc=True),
            Pin(num='A14',name='A.A4',func=Pin.TRISTATE,do_erc=True),
            Pin(num='B14',name='A.A9',func=Pin.TRISTATE,do_erc=True),
            Pin(num='C14',name='A.A15',func=Pin.TRISTATE,do_erc=True),
            Pin(num='D14',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='R14',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='T14',name='B.D4',func=Pin.TRISTATE,do_erc=True),
            Pin(num='U14',name='B.D6',func=Pin.TRISTATE,do_erc=True),
            Pin(num='V14',name='B.D8',func=Pin.TRISTATE,do_erc=True),
            Pin(num='A15',name='A.A7',func=Pin.TRISTATE,do_erc=True),
            Pin(num='B15',name='A.A11',func=Pin.TRISTATE,do_erc=True),
            Pin(num='C15',name='A.A17',func=Pin.TRISTATE,do_erc=True),
            Pin(num='D15',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='E15',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='F15',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='G15',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='H15',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='J15',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='K15',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='L15',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='M15',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='N15',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='P15',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='R15',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='T15',name='B.D1',func=Pin.TRISTATE,do_erc=True),
            Pin(num='U15',name='B.D3',func=Pin.TRISTATE,do_erc=True),
            Pin(num='V15',name='B.D5',func=Pin.TRISTATE,do_erc=True),
            Pin(num='A16',name='A.A10',func=Pin.TRISTATE,do_erc=True),
            Pin(num='B16',name='A.A14',func=Pin.TRISTATE,do_erc=True),
            Pin(num='C16',name='A.A19',func=Pin.TRISTATE,do_erc=True),
            Pin(num='D16',name='A.A22',func=Pin.TRISTATE,do_erc=True),
            Pin(num='E16',name='A.A24',func=Pin.TRISTATE,do_erc=True),
            Pin(num='F16',name='A.A27',func=Pin.TRISTATE,do_erc=True),
            Pin(num='G16',name='A.A31',func=Pin.TRISTATE,do_erc=True),
            Pin(num='H16',name='A.D28',func=Pin.TRISTATE,do_erc=True),
            Pin(num='J16',name='A.D24',func=Pin.TRISTATE,do_erc=True),
            Pin(num='K16',name='A.D20',func=Pin.TRISTATE,do_erc=True),
            Pin(num='L16',name='A.D16',func=Pin.TRISTATE,do_erc=True),
            Pin(num='M16',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='N16',name='A.D11',func=Pin.TRISTATE,do_erc=True),
            Pin(num='P16',name='A.D7',func=Pin.TRISTATE,do_erc=True),
            Pin(num='R16',name='A.D5',func=Pin.TRISTATE,do_erc=True),
            Pin(num='T16',name='A.D2',func=Pin.TRISTATE,do_erc=True),
            Pin(num='U16',name='B.D0',func=Pin.TRISTATE,do_erc=True),
            Pin(num='V16',name='B.D2',func=Pin.TRISTATE,do_erc=True),
            Pin(num='A17',name='A.A13',func=Pin.TRISTATE,do_erc=True),
            Pin(num='B17',name='A.A18',func=Pin.TRISTATE,do_erc=True),
            Pin(num='C17',name='A.A21',func=Pin.TRISTATE,do_erc=True),
            Pin(num='D17',name='A.A25',func=Pin.TRISTATE,do_erc=True),
            Pin(num='E17',name='A.A28',func=Pin.TRISTATE,do_erc=True),
            Pin(num='F17',name='A.A30',func=Pin.TRISTATE,do_erc=True),
            Pin(num='G17',name='A.D30',func=Pin.TRISTATE,do_erc=True),
            Pin(num='H17',name='A.D27',func=Pin.TRISTATE,do_erc=True),
            Pin(num='J17',name='A.D25',func=Pin.TRISTATE,do_erc=True),
            Pin(num='K17',name='A.D21',func=Pin.TRISTATE,do_erc=True),
            Pin(num='L17',name='A.D18',func=Pin.TRISTATE,do_erc=True),
            Pin(num='M17',name='A.DENB',do_erc=True),
            Pin(num='N17',name='A.D14',func=Pin.TRISTATE,do_erc=True),
            Pin(num='P17',name='A.D12',func=Pin.TRISTATE,do_erc=True),
            Pin(num='R17',name='A.D9',func=Pin.TRISTATE,do_erc=True),
            Pin(num='T17',name='A.D6',func=Pin.TRISTATE,do_erc=True),
            Pin(num='U17',name='A.D3',func=Pin.TRISTATE,do_erc=True),
            Pin(num='V17',name='A.D0',func=Pin.TRISTATE,do_erc=True),
            Pin(num='A18',name='A.A16',func=Pin.TRISTATE,do_erc=True),
            Pin(num='B18',name='A.A20',func=Pin.TRISTATE,do_erc=True),
            Pin(num='C18',name='A.A23',func=Pin.TRISTATE,do_erc=True),
            Pin(num='D18',name='A.A26',func=Pin.TRISTATE,do_erc=True),
            Pin(num='E18',name='A.A29',func=Pin.TRISTATE,do_erc=True),
            Pin(num='F18',name='A.D31',func=Pin.TRISTATE,do_erc=True),
            Pin(num='G18',name='A.D29',func=Pin.TRISTATE,do_erc=True),
            Pin(num='H18',name='A.D26',func=Pin.TRISTATE,do_erc=True),
            Pin(num='J18',name='A.D23',func=Pin.TRISTATE,do_erc=True),
            Pin(num='K18',name='A.D22',func=Pin.TRISTATE,do_erc=True),
            Pin(num='L18',name='A.D19',func=Pin.TRISTATE,do_erc=True),
            Pin(num='M18',name='A.D17',func=Pin.TRISTATE,do_erc=True),
            Pin(num='N18',name='A.D15',func=Pin.TRISTATE,do_erc=True),
            Pin(num='P18',name='A.D13',func=Pin.TRISTATE,do_erc=True),
            Pin(num='R18',name='A.D10',func=Pin.TRISTATE,do_erc=True),
            Pin(num='T18',name='A.D8',func=Pin.TRISTATE,do_erc=True),
            Pin(num='U18',name='A.D4',func=Pin.TRISTATE,do_erc=True),
            Pin(num='V18',name='A.D1',func=Pin.TRISTATE,do_erc=True)]),
        Part(name='TMS320LF2406PZ',dest=TEMPLATE,tool=SKIDL,keywords='16BIT DSP TMS320 Obsolete',description='16bit DSP Controller 32Kx16B Flash 2.5Kx16B RAM, Obsolete NRND, PQFP-100',ref_prefix='U',num_units=1,fplist=['PQFP-100*'],do_erc=True,pins=[
            Pin(num='1',name='TRST',do_erc=True),
            Pin(num='2',name='TDIRB/IOPF4',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='5',name='T4PWM/T4CMP/IOPF3',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='PDPINTA',do_erc=True),
            Pin(num='7',name='T3PWM/T3CMP/IOPF2',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='PLLF2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='9',name='PLLF',func=Pin.PASSIVE,do_erc=True),
            Pin(num='10',name='PLLVcca',func=Pin.PASSIVE,do_erc=True),
            Pin(num='20',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='30',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='40',name='VccPROG_5V',func=Pin.PASSIVE,do_erc=True),
            Pin(num='50',name='CANTX/IOPC6',func=Pin.BIDIR,do_erc=True),
            Pin(num='60',name='CAP4/QEP3/IOPE7',func=Pin.BIDIR,do_erc=True),
            Pin(num='70',name='ADCIN04',do_erc=True),
            Pin(num='80',name='ADCIN08',do_erc=True),
            Pin(num='90',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='11',name='TDIRA/IOPB6',func=Pin.BIDIR,do_erc=True),
            Pin(num='21',name='SPI_SIMO/IOPC2',func=Pin.BIDIR,do_erc=True),
            Pin(num='31',name='PWM5/IOPB2',func=Pin.BIDIR,do_erc=True),
            Pin(num='41',name='PWM9/IOPE3',func=Pin.BIDIR,do_erc=True),
            Pin(num='51',name='CLKOUT/IOPE0',func=Pin.BIDIR,do_erc=True),
            Pin(num='61',name='EMU0',func=Pin.BIDIR,do_erc=True),
            Pin(num='71',name='ADCIN13',do_erc=True),
            Pin(num='81',name='VrefLO',func=Pin.PASSIVE,do_erc=True),
            Pin(num='91',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='12',name='T1PWM/T1CMP/IOPB4',func=Pin.BIDIR,do_erc=True),
            Pin(num='22',name='SPI_SOMI/IOPC3',func=Pin.BIDIR,do_erc=True),
            Pin(num='32',name='PWM11/IOPE5',func=Pin.BIDIR,do_erc=True),
            Pin(num='52',name='CAP3/IOPA5',func=Pin.BIDIR,do_erc=True),
            Pin(num='62',name='EMU1/OFF-',func=Pin.BIDIR,do_erc=True),
            Pin(num='72',name='ADCIN03',do_erc=True),
            Pin(num='82',name='VrefHI',func=Pin.PASSIVE,do_erc=True),
            Pin(num='92',name='IOPF6',func=Pin.BIDIR,do_erc=True),
            Pin(num='13',name='T2PWM/T2CMP/IOPB5',func=Pin.BIDIR,do_erc=True),
            Pin(num='23',name='SPI_TE-/IOPC5',func=Pin.BIDIR,do_erc=True),
            Pin(num='33',name='PWM4/IOPB1',func=Pin.BIDIR,do_erc=True),
            Pin(num='43',name='PWM8/IOPE2',func=Pin.BIDIR,do_erc=True),
            Pin(num='53',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='63',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='73',name='ADCIN12',do_erc=True),
            Pin(num='83',name='VCCA',func=Pin.PWRIN,do_erc=True),
            Pin(num='93',name='RESET',do_erc=True),
            Pin(num='14',name='IOPC0',func=Pin.BIDIR,do_erc=True),
            Pin(num='24',name='SPI_CLK/IOPC4',func=Pin.BIDIR,do_erc=True),
            Pin(num='34',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='54',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='64',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='74',name='ADCIN02',do_erc=True),
            Pin(num='84',name='VSSA',func=Pin.PWRIN,do_erc=True),
            Pin(num='94',name='TCK',do_erc=True),
            Pin(num='15',name='XINT2/IOPD0',func=Pin.BIDIR,do_erc=True),
            Pin(num='25',name='TMS2',do_erc=True),
            Pin(num='35',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='45',name='PWM7/IOPE1',func=Pin.BIDIR,do_erc=True),
            Pin(num='55',name='CAP2/QEP2/IOPA4',func=Pin.BIDIR,do_erc=True),
            Pin(num='65',name='ADCIN15',do_erc=True),
            Pin(num='75',name='ADCIN11',do_erc=True),
            Pin(num='85',name='BIO-/IOPC1',func=Pin.BIDIR,do_erc=True),
            Pin(num='95',name='PDPINTB',do_erc=True),
            Pin(num='16',name='XINT1/IOPA2',func=Pin.BIDIR,do_erc=True),
            Pin(num='26',name='TCLKINA/IOPB7',func=Pin.PASSIVE,do_erc=True),
            Pin(num='36',name='PWM3/IOPB0',func=Pin.BIDIR,do_erc=True),
            Pin(num='46',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='56',name='CAP5/QEP4/IOPF0',func=Pin.PASSIVE,do_erc=True),
            Pin(num='66',name='ADCIN07',do_erc=True),
            Pin(num='76',name='ADCIN10',do_erc=True),
            Pin(num='86',name='~BOOT_EN~/XF',func=Pin.OUTPUT,do_erc=True),
            Pin(num='96',name='TDI',do_erc=True),
            Pin(num='17',name='TXD/IOPA0',func=Pin.BIDIR,do_erc=True),
            Pin(num='27',name='PWM12/IOPE6',func=Pin.BIDIR,do_erc=True),
            Pin(num='37',name='PWM2/IOPA7',func=Pin.BIDIR,do_erc=True),
            Pin(num='47',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='57',name='CAP1/QEP1/IOPA3',func=Pin.BIDIR,do_erc=True),
            Pin(num='67',name='ADCIN06',do_erc=True),
            Pin(num='77',name='ADCIN01',do_erc=True),
            Pin(num='87',name='XTAL/CLKIN',do_erc=True),
            Pin(num='97',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='18',name='RXD/IOPA1',func=Pin.BIDIR,do_erc=True),
            Pin(num='28',name='PWM6/IOPB3',func=Pin.BIDIR,do_erc=True),
            Pin(num='38',name='PWM10/IOPE4',func=Pin.BIDIR,do_erc=True),
            Pin(num='48',name='CAP6/IOPF1',func=Pin.BIDIR,do_erc=True),
            Pin(num='58',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='68',name='ADCIN14',do_erc=True),
            Pin(num='78',name='ADCIN09',do_erc=True),
            Pin(num='88',name='XTAL2',func=Pin.OUTPUT,do_erc=True),
            Pin(num='98',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='19',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='29',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='39',name='PWM1/IOPA6',func=Pin.BIDIR,do_erc=True),
            Pin(num='49',name='CANRX/IOPC7',func=Pin.BIDIR,do_erc=True),
            Pin(num='59',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='69',name='ADCIN05',do_erc=True),
            Pin(num='79',name='ADCIN00',do_erc=True),
            Pin(num='89',name='TCLKINB/IOPF5',func=Pin.BIDIR,do_erc=True),
            Pin(num='99',name='TDO',func=Pin.OUTPUT,do_erc=True),
            Pin(num='100',name='TMS',do_erc=True)])])


from skidl import Pin, Part, SchLib, SKIDL, TEMPLATE

SKIDL_lib_version = '0.0.1'

microchip_pic24mcu = SchLib(tool=SKIDL).add_parts(*[
        Part(name='PIC24FV32KA304-I/PT',dest=TEMPLATE,tool=SKIDL,keywords='Flash-Based 16-Bit Microcontroller',description='32K Flash, 2048B SRAM, 512B EEPROM, nanoWatt XLP, 12-bit ADC, PWM, RTCC, TQFP44',ref_prefix='IC',num_units=1,fplist=['TQFP*10x10mm*Pitch0.8mm*'],do_erc=True,pins=[
            Pin(num='1',name='RB9/DA1/T1CK/~U1RTS~/CTED4/CN21',func=Pin.BIDIR,do_erc=True),
            Pin(num='2',name='U1RX/CN18/RC6',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='U1TX/CN17/RC7',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='OC2/CN20/RC8',func=Pin.BIDIR,do_erc=True),
            Pin(num='5',name='IC2/CTED7/CN19/RC9',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='RA7/IC1/CTED3/CN9',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='RA6/C2OUT/OC1/CTED1/INT2/CN8',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='RB10/PGED2/SDI1/CTED11/CN16',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='RB11/PGEC2/SCK1/CTED9/CN15',func=Pin.BIDIR,do_erc=True),
            Pin(num='10',name='RB12/N12/HLVDIN/CTED2/CN14',func=Pin.BIDIR,do_erc=True),
            Pin(num='20',name='RA1/CVREF-/VREF-/AN1/CN3',func=Pin.BIDIR,do_erc=True),
            Pin(num='30',name='RA2/OSCI/AN13/CLKI/CN30',func=Pin.BIDIR,do_erc=True),
            Pin(num='40',name='Vdd',func=Pin.PWRIN,do_erc=True),
            Pin(num='11',name='RB13/AN11/SDO1/CTPLS/CN13',func=Pin.BIDIR,do_erc=True),
            Pin(num='21',name='RB0/PGED1/AN2/ULPWU/CTCMP/C1IND/C2INB/C3IND/U2TX/CN4',func=Pin.BIDIR,do_erc=True),
            Pin(num='31',name='RA3/OSCO/AN14/CLKO/CN29',func=Pin.BIDIR,do_erc=True),
            Pin(num='41',name='RB5/PGED3/ASDA1*/CN27',func=Pin.BIDIR,do_erc=True),
            Pin(num='12',name='RA10/OC3/CN35',func=Pin.BIDIR,do_erc=True),
            Pin(num='22',name='RB1/PGEC1/AN3/C1INC/C2INA/U2RX/CTED12/CN5',func=Pin.BIDIR,do_erc=True),
            Pin(num='32',name='RA8/OCFB/CN33',func=Pin.BIDIR,do_erc=True),
            Pin(num='42',name='RB6/PGEC3/ASCL1*/CN24',func=Pin.BIDIR,do_erc=True),
            Pin(num='13',name='RA11/IC3/CTED8/CN36',func=Pin.BIDIR,do_erc=True),
            Pin(num='23',name='RB2/AN4/C1INB/C2IND/SDA2/T5CK/T4CK/CTED13/CN6',func=Pin.BIDIR,do_erc=True),
            Pin(num='33',name='RB4/SOSCI/AN15/~U2RTS~/CN1',func=Pin.BIDIR,do_erc=True),
            Pin(num='43',name='RB7/INT0/CN23',func=Pin.BIDIR,do_erc=True),
            Pin(num='14',name='RB14/CVREF/AN10/C3INB/RTCC/C1OUT/OCFA/CTED5/INT1/CN12',func=Pin.BIDIR,do_erc=True),
            Pin(num='24',name='RB3/AN5/C1INA/C2INC/SCL2/CN7',func=Pin.BIDIR,do_erc=True),
            Pin(num='34',name='RA4/SOSCO/SCLKI/U2CTS/CN0',func=Pin.BIDIR,do_erc=True),
            Pin(num='44',name='RB8/SCL1/U1CTS/C3OUT/CTED10',func=Pin.BIDIR,do_erc=True),
            Pin(num='15',name='RB15/AN9/C3INA/T3CK/T2CK/REFO/SS1/CTED6/CN11',func=Pin.BIDIR,do_erc=True),
            Pin(num='25',name='AN6/CN32/RC0',func=Pin.BIDIR,do_erc=True),
            Pin(num='35',name='RA9/~SS2~/CN34',func=Pin.BIDIR,do_erc=True),
            Pin(num='16',name='Vss/AVss',func=Pin.PWRIN,do_erc=True),
            Pin(num='26',name='AN7/CN31/RC1',func=Pin.BIDIR,do_erc=True),
            Pin(num='36',name='SDI2/CN28/RC3',func=Pin.BIDIR,do_erc=True),
            Pin(num='17',name='AVdd/Vdd',func=Pin.PWRIN,do_erc=True),
            Pin(num='27',name='AN8/CN10/RC2',func=Pin.BIDIR,do_erc=True),
            Pin(num='37',name='SDO2/CN25/RC4',func=Pin.BIDIR,do_erc=True),
            Pin(num='18',name='RA5/~MCLR~/Vpp',func=Pin.BIDIR,do_erc=True),
            Pin(num='28',name='Vdd',func=Pin.PWRIN,do_erc=True),
            Pin(num='38',name='SCK2/CN26/RC5',func=Pin.BIDIR,do_erc=True),
            Pin(num='19',name='RA0/VREF+/CVREF+/AN0/C3INC/CN2',func=Pin.BIDIR,do_erc=True),
            Pin(num='29',name='Vss',func=Pin.PWRIN,do_erc=True),
            Pin(num='39',name='Vss',func=Pin.PWRIN,do_erc=True)])])


from skidl import *
import os
import os.path

def convert_libs(from_dir, to_dir):
    lib_files = [l for l in os.listdir(from_dir) if l.endswith(lib_suffixes[KICAD])]
    for lib_file in lib_files:
        print(lib_file)
        basename = os.path.splitext(lib_file)[0]
        lib = SchLib(os.path.join(from_dir, lib_file), tool=KICAD)
        lib.export(libname=basename, file=os.path.join(to_dir, basename + lib_suffixes[SKIDL]))

if __name__ == '__main__':
    import skidl.libs
    for lib_dir in lib_search_paths[KICAD]:
        convert_libs(lib_dir, skidl.libs.__path__[0])



from skidl import Pin, Part, SchLib, SKIDL, TEMPLATE

SKIDL_lib_version = '0.0.1'

ftdi = SchLib(tool=SKIDL).add_parts(*[
        Part(name='FT200XD',dest=TEMPLATE,tool=SKIDL,keywords='FTDI USB I2C Interface Converter',description='Full Speed USB to I2C Bridge, DFN-10',ref_prefix='U',num_units=1,fplist=['DFN*'],do_erc=True,pins=[
            Pin(num='1',name='USBDM',do_erc=True),
            Pin(num='2',name='~RESET',do_erc=True),
            Pin(num='3',name='3V3OUT',func=Pin.PWROUT,do_erc=True),
            Pin(num='4',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='5',name='CBUS0',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='SCL',do_erc=True),
            Pin(num='7',name='VCCIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='SDA',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='10',name='USBDP',do_erc=True),
            Pin(num='11',name='GND',func=Pin.PWRIN,do_erc=True)]),
        Part(name='FT201XQ',dest=TEMPLATE,tool=SKIDL,keywords='FTDI USB I2C interface Converter',description='Full Speed USB to I2C Bridge, QFN-10',ref_prefix='U',num_units=1,fplist=['QFN*'],do_erc=True,pins=[
            Pin(num='1',name='VCCIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='SDA',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='CBUS4',func=Pin.BIDIR,do_erc=True),
            Pin(num='5',name='CBUS2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='USBDP',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='USBDM',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='3V3OUT',func=Pin.PWROUT,do_erc=True),
            Pin(num='9',name='~RESET',do_erc=True),
            Pin(num='10',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='11',name='CBUS1',func=Pin.BIDIR,do_erc=True),
            Pin(num='12',name='CBUS0',func=Pin.BIDIR,do_erc=True),
            Pin(num='13',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='CBUS3',func=Pin.BIDIR,do_erc=True),
            Pin(num='15',name='CBUS5',func=Pin.BIDIR,do_erc=True),
            Pin(num='16',name='SCL',do_erc=True),
            Pin(num='17',name='GND',func=Pin.PWRIN,do_erc=True)]),
        Part(name='FT201XS',dest=TEMPLATE,tool=SKIDL,keywords='FTDI USB I2C Interface Converter',description='Full Speed USB to I2C Bridge, SSOP-16',ref_prefix='U',num_units=1,fplist=['SSOP*'],do_erc=True,pins=[
            Pin(num='1',name='CBUS5',func=Pin.BIDIR,do_erc=True),
            Pin(num='2',name='SCL',do_erc=True),
            Pin(num='3',name='VCCIO',func=Pin.PWROUT,do_erc=True),
            Pin(num='4',name='SDA',func=Pin.BIDIR,do_erc=True),
            Pin(num='5',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='6',name='CBUS4',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='CBUS2',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='USBDP',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='USBDM',func=Pin.BIDIR,do_erc=True),
            Pin(num='10',name='3V3OUT',func=Pin.PWROUT,do_erc=True),
            Pin(num='11',name='~RESET',do_erc=True),
            Pin(num='12',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='13',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='CBUS1',func=Pin.BIDIR,do_erc=True),
            Pin(num='15',name='CBUS0',func=Pin.BIDIR,do_erc=True),
            Pin(num='16',name='CBUS3',func=Pin.BIDIR,do_erc=True)]),
        Part(name='FT220XQ',dest=TEMPLATE,tool=SKIDL,keywords='FTDI USB SPI FT1248 Interface Converter',description='Full Speed USB to 4-Bit SPI / FT1248 Bridge, QFN-16',ref_prefix='U',num_units=1,fplist=['QFN*'],do_erc=True,pins=[
            Pin(num='1',name='VCCIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='MIOSI1',do_erc=True),
            Pin(num='3',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='MIOSI3',do_erc=True),
            Pin(num='5',name='MISO',do_erc=True),
            Pin(num='6',name='USBDP',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='USBDM',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='3V3OUT',func=Pin.PWROUT,do_erc=True),
            Pin(num='9',name='~RESET',do_erc=True),
            Pin(num='10',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='11',name='~CS',do_erc=True),
            Pin(num='12',name='CLK',do_erc=True),
            Pin(num='13',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='CBUS3',func=Pin.BIDIR,do_erc=True),
            Pin(num='15',name='MIOSI0',do_erc=True),
            Pin(num='16',name='MIOSI2',do_erc=True),
            Pin(num='17',name='GND',func=Pin.PWRIN,do_erc=True)]),
        Part(name='FT220XS',dest=TEMPLATE,tool=SKIDL,keywords='FTDI USB SPI FT1248 Interface Converter',description='Full Speed USB to 4-Bit SPI / FT1248 Bridge, SSOP-16',ref_prefix='U',num_units=1,fplist=['SSOP*'],do_erc=True,pins=[
            Pin(num='1',name='MIOSI0',do_erc=True),
            Pin(num='2',name='MIOSI2',do_erc=True),
            Pin(num='3',name='VCCIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='MIOSI1',do_erc=True),
            Pin(num='5',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='6',name='MIOSI3',do_erc=True),
            Pin(num='7',name='MISO',do_erc=True),
            Pin(num='8',name='USBDP',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='USBDM',func=Pin.BIDIR,do_erc=True),
            Pin(num='10',name='3V3OUT',func=Pin.PWROUT,do_erc=True),
            Pin(num='11',name='~RESET',do_erc=True),
            Pin(num='12',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='13',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='~CS',do_erc=True),
            Pin(num='15',name='CLK',do_erc=True),
            Pin(num='16',name='CBUS3',func=Pin.BIDIR,do_erc=True)]),
        Part(name='FT221XQ',dest=TEMPLATE,tool=SKIDL,keywords='FTDI USB SPI FT1248 interface converter',description='Full Speed USB to 8-Bit SPI / FT1248 Bridge, QFN-20',ref_prefix='U',num_units=1,fplist=['QFN*'],do_erc=True,pins=[
            Pin(num='1',name='MIOSI1',do_erc=True),
            Pin(num='2',name='MIOSI7',do_erc=True),
            Pin(num='3',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='MIOSI5',do_erc=True),
            Pin(num='5',name='MIOSI6',do_erc=True),
            Pin(num='6',name='MIOSI3',do_erc=True),
            Pin(num='7',name='MISO',do_erc=True),
            Pin(num='8',name='USBDP',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='USBDM',func=Pin.BIDIR,do_erc=True),
            Pin(num='10',name='3V3OUT',func=Pin.PWROUT,do_erc=True),
            Pin(num='20',name='VCCIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='11',name='~RESET',do_erc=True),
            Pin(num='21',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='12',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='13',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='~CS',do_erc=True),
            Pin(num='15',name='CLK',do_erc=True),
            Pin(num='16',name='CBUS3',func=Pin.BIDIR,do_erc=True),
            Pin(num='17',name='MIOSI0',do_erc=True),
            Pin(num='18',name='MIOSI4',do_erc=True),
            Pin(num='19',name='MIOSI2',do_erc=True)]),
        Part(name='FT221XS',dest=TEMPLATE,tool=SKIDL,keywords='FTDI USB SPI FT1248 interface converter',description='Full Speed USB to 8-Bit SPI / FT1248 Bridge, SSOP-20',ref_prefix='U',num_units=1,fplist=['SSOP*'],do_erc=True,pins=[
            Pin(num='1',name='MIOSI4',do_erc=True),
            Pin(num='2',name='MIOSI2',do_erc=True),
            Pin(num='3',name='VCCIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='MIOSI1',do_erc=True),
            Pin(num='5',name='MIOSI7',do_erc=True),
            Pin(num='6',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='7',name='MIOSI5',do_erc=True),
            Pin(num='8',name='MIOSI6',do_erc=True),
            Pin(num='9',name='MIOSI3',do_erc=True),
            Pin(num='10',name='MISO',do_erc=True),
            Pin(num='20',name='MIOSI0',do_erc=True),
            Pin(num='11',name='USBDP',func=Pin.BIDIR,do_erc=True),
            Pin(num='12',name='USBDM',func=Pin.BIDIR,do_erc=True),
            Pin(num='13',name='3V3OUT',func=Pin.PWROUT,do_erc=True),
            Pin(num='14',name='~RESET',do_erc=True),
            Pin(num='15',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='17',name='~CS',do_erc=True),
            Pin(num='18',name='CLK',do_erc=True),
            Pin(num='19',name='CBUS3',func=Pin.BIDIR,do_erc=True)]),
        Part(name='FT2232D',dest=TEMPLATE,tool=SKIDL,keywords='USB Double UART FIFO',description='Full Speed Double Channel USB UART/FIFO, LQFP-48',ref_prefix='U',num_units=1,fplist=['LQFP*7x7mm*Pitch0.5mm*'],do_erc=True,pins=[
            Pin(num='1',name='EECLK',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='EEDATA',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='~RESET',do_erc=True),
            Pin(num='5',name='~RSTOUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='6',name='3V3OUT',func=Pin.PWROUT,do_erc=True),
            Pin(num='7',name='USBDP',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='USBDM',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='10',name='SI/WUA',func=Pin.BIDIR,do_erc=True),
            Pin(num='20',name='ADBUS4',func=Pin.BIDIR,do_erc=True),
            Pin(num='30',name='BCBUS0',func=Pin.OUTPUT,do_erc=True),
            Pin(num='40',name='BDBUS0',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='ACBUS3',func=Pin.BIDIR,do_erc=True),
            Pin(num='21',name='ADBUS3',func=Pin.BIDIR,do_erc=True),
            Pin(num='31',name='VCCIOB',func=Pin.PWRIN,do_erc=True),
            Pin(num='41',name='~PWREN',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='ACBUS2',func=Pin.BIDIR,do_erc=True),
            Pin(num='22',name='ADBUS2',func=Pin.BIDIR,do_erc=True),
            Pin(num='32',name='BDBUS7',func=Pin.OUTPUT,do_erc=True),
            Pin(num='42',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='13',name='ACBUS1',func=Pin.BIDIR,do_erc=True),
            Pin(num='23',name='ADBUS1',func=Pin.BIDIR,do_erc=True),
            Pin(num='33',name='BDBUS6',func=Pin.OUTPUT,do_erc=True),
            Pin(num='43',name='XTIN',do_erc=True),
            Pin(num='14',name='VCCIOA',func=Pin.PWRIN,do_erc=True),
            Pin(num='24',name='ADBUS0',func=Pin.BIDIR,do_erc=True),
            Pin(num='34',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='44',name='XTOUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='15',name='ACBUS0',func=Pin.BIDIR,do_erc=True),
            Pin(num='25',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='35',name='BDBUS5',func=Pin.OUTPUT,do_erc=True),
            Pin(num='45',name='AGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='ADBUS7',func=Pin.BIDIR,do_erc=True),
            Pin(num='26',name='SI/WUB',func=Pin.OUTPUT,do_erc=True),
            Pin(num='36',name='BDBUS4',func=Pin.OUTPUT,do_erc=True),
            Pin(num='46',name='AVCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='17',name='ADBUS6',func=Pin.BIDIR,do_erc=True),
            Pin(num='27',name='BCBUS3',func=Pin.OUTPUT,do_erc=True),
            Pin(num='37',name='BDBUS3',func=Pin.OUTPUT,do_erc=True),
            Pin(num='47',name='TEST',do_erc=True),
            Pin(num='18',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='28',name='BCBUS2',func=Pin.OUTPUT,do_erc=True),
            Pin(num='38',name='BDBUS2',func=Pin.OUTPUT,do_erc=True),
            Pin(num='48',name='EECS',func=Pin.BIDIR,do_erc=True),
            Pin(num='19',name='ADBUS5',func=Pin.BIDIR,do_erc=True),
            Pin(num='29',name='BCBUS1',func=Pin.OUTPUT,do_erc=True),
            Pin(num='39',name='BDBUS1',func=Pin.OUTPUT,do_erc=True)]),
        Part(name='FT2232H',dest=TEMPLATE,tool=SKIDL,keywords='USB Double UART FIFO',description='FT2232Hx, Hi Speed Double Channel USB UART/FIFO, LQFP/QFN-64',ref_prefix='U',num_units=1,fplist=['QFN*', 'LQFP*'],do_erc=True,pins=[
            Pin(num='1',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='OSCI',do_erc=True),
            Pin(num='3',name='OSCO',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='VPHY',func=Pin.PWRIN,do_erc=True),
            Pin(num='5',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='6',name='REF',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='DM',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='DP',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='VPLL',do_erc=True),
            Pin(num='10',name='AGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='20',name='VCCIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='30',name='ACBUS4',func=Pin.BIDIR,do_erc=True),
            Pin(num='40',name='BDBUS2',func=Pin.BIDIR,do_erc=True),
            Pin(num='50',name='VREGIN',func=Pin.PWRIN,do_erc=True),
            Pin(num='60',name='~PWREN',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='21',name='ADBUS4',func=Pin.BIDIR,do_erc=True),
            Pin(num='31',name='VCCIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='41',name='BDBUS3',func=Pin.BIDIR,do_erc=True),
            Pin(num='51',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='61',name='EEDATA',func=Pin.BIDIR,do_erc=True),
            Pin(num='12',name='VCORE',func=Pin.PWRIN,do_erc=True),
            Pin(num='22',name='ADBUS5',func=Pin.BIDIR,do_erc=True),
            Pin(num='32',name='ACBUS5',func=Pin.BIDIR,do_erc=True),
            Pin(num='42',name='VCCIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='52',name='BCBUS1',func=Pin.BIDIR,do_erc=True),
            Pin(num='62',name='EECLK',func=Pin.OUTPUT,do_erc=True),
            Pin(num='13',name='TEST',do_erc=True),
            Pin(num='23',name='ADBUS6',func=Pin.BIDIR,do_erc=True),
            Pin(num='33',name='ACBUS6',func=Pin.BIDIR,do_erc=True),
            Pin(num='43',name='BDBUS4',func=Pin.BIDIR,do_erc=True),
            Pin(num='53',name='BCBUS2',func=Pin.BIDIR,do_erc=True),
            Pin(num='63',name='EECS',func=Pin.OUTPUT,do_erc=True),
            Pin(num='14',name='~RESET',do_erc=True),
            Pin(num='24',name='ADBUS7',func=Pin.BIDIR,do_erc=True),
            Pin(num='34',name='ACBUS7',func=Pin.BIDIR,do_erc=True),
            Pin(num='44',name='BDBUS5',func=Pin.BIDIR,do_erc=True),
            Pin(num='54',name='BCBUS3',func=Pin.BIDIR,do_erc=True),
            Pin(num='64',name='VCORE',func=Pin.PWRIN,do_erc=True),
            Pin(num='15',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='25',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='35',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='45',name='BDBUS6',func=Pin.BIDIR,do_erc=True),
            Pin(num='55',name='BCBUS4',func=Pin.BIDIR,do_erc=True),
            Pin(num='16',name='ADBUS0',func=Pin.BIDIR,do_erc=True),
            Pin(num='26',name='ACBUS0',func=Pin.BIDIR,do_erc=True),
            Pin(num='36',name='~SUSPEND',func=Pin.OUTPUT,do_erc=True),
            Pin(num='46',name='BDBUS7',func=Pin.BIDIR,do_erc=True),
            Pin(num='56',name='VCCIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='17',name='ADBUS1',func=Pin.BIDIR,do_erc=True),
            Pin(num='27',name='ACBUS1',func=Pin.BIDIR,do_erc=True),
            Pin(num='37',name='VCORE',func=Pin.PWRIN,do_erc=True),
            Pin(num='47',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='57',name='BCBUS5',func=Pin.BIDIR,do_erc=True),
            Pin(num='18',name='ADBUS2',func=Pin.BIDIR,do_erc=True),
            Pin(num='28',name='ACBUS2',func=Pin.BIDIR,do_erc=True),
            Pin(num='38',name='BDBUS0',func=Pin.BIDIR,do_erc=True),
            Pin(num='48',name='BCBUS0',func=Pin.BIDIR,do_erc=True),
            Pin(num='58',name='BCBUS6',func=Pin.BIDIR,do_erc=True),
            Pin(num='19',name='ADBUS3',func=Pin.BIDIR,do_erc=True),
            Pin(num='29',name='ACBUS3',func=Pin.BIDIR,do_erc=True),
            Pin(num='39',name='BDBUS1',func=Pin.BIDIR,do_erc=True),
            Pin(num='49',name='VREGOUT',func=Pin.PWROUT,do_erc=True),
            Pin(num='59',name='BCBUS7',func=Pin.BIDIR,do_erc=True)]),
        Part(name='FT230XQ',dest=TEMPLATE,tool=SKIDL,keywords='FTDI USB UART interface converter',description='Full Speed USB to Basic UART, QFN-16',ref_prefix='U',num_units=1,fplist=['QFN*'],do_erc=True,pins=[
            Pin(num='1',name='VCCIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='RXD',do_erc=True),
            Pin(num='3',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='~CTS',do_erc=True),
            Pin(num='5',name='CBUS2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='USBDP',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='USBDM',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='3V3OUT',func=Pin.PWROUT,do_erc=True),
            Pin(num='9',name='~RESET',do_erc=True),
            Pin(num='10',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='11',name='CBUS1',func=Pin.BIDIR,do_erc=True),
            Pin(num='12',name='CBUS0',func=Pin.BIDIR,do_erc=True),
            Pin(num='13',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='CBUS3',func=Pin.BIDIR,do_erc=True),
            Pin(num='15',name='TXD',func=Pin.OUTPUT,do_erc=True),
            Pin(num='16',name='~RTS',do_erc=True),
            Pin(num='17',name='GND',func=Pin.PWRIN,do_erc=True)]),
        Part(name='FT230XS',dest=TEMPLATE,tool=SKIDL,keywords='FTDI USB UART interface converter',description='Full Speed USB to Basic UART, SSOP-16',ref_prefix='U',num_units=1,fplist=['SSOP*'],do_erc=True,pins=[
            Pin(num='1',name='TXD',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='~RTS',do_erc=True),
            Pin(num='3',name='VCCIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='RXD',do_erc=True),
            Pin(num='5',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='6',name='~CTS',do_erc=True),
            Pin(num='7',name='CBUS2',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='USBDP',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='USBDM',func=Pin.BIDIR,do_erc=True),
            Pin(num='10',name='3V3OUT',func=Pin.PWROUT,do_erc=True),
            Pin(num='11',name='~RESET',do_erc=True),
            Pin(num='12',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='13',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='CBUS1',func=Pin.BIDIR,do_erc=True),
            Pin(num='15',name='CBUS0',func=Pin.BIDIR,do_erc=True),
            Pin(num='16',name='CBUS3',func=Pin.BIDIR,do_erc=True)]),
        Part(name='FT231XQ',dest=TEMPLATE,tool=SKIDL,keywords='FTDI USB UART interface converter',description='Full Speed USB to Full Handshake UART, QFN-20',ref_prefix='U',num_units=1,fplist=['QFN*'],do_erc=True,pins=[
            Pin(num='1',name='RXD',do_erc=True),
            Pin(num='2',name='~RI',do_erc=True),
            Pin(num='3',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='~DSR',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='~DCD',do_erc=True),
            Pin(num='6',name='~CTS',do_erc=True),
            Pin(num='7',name='CBUS2',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='USBDP',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='USBDM',func=Pin.BIDIR,do_erc=True),
            Pin(num='10',name='3V3OUT',func=Pin.PWROUT,do_erc=True),
            Pin(num='20',name='VCCIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='11',name='~RESET',do_erc=True),
            Pin(num='21',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='12',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='13',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='CBUS1',func=Pin.BIDIR,do_erc=True),
            Pin(num='15',name='CBUS0',func=Pin.BIDIR,do_erc=True),
            Pin(num='16',name='CBUS3',func=Pin.BIDIR,do_erc=True),
            Pin(num='17',name='TXD',func=Pin.OUTPUT,do_erc=True),
            Pin(num='18',name='~DTR',func=Pin.OUTPUT,do_erc=True),
            Pin(num='19',name='~RTS',do_erc=True)]),
        Part(name='FT231XS',dest=TEMPLATE,tool=SKIDL,keywords='FTDI USB UART interface converter',description='Full Speed USB to Full Handshake UART, SSOP-20',ref_prefix='U',num_units=1,fplist=['SSOP*'],do_erc=True,pins=[
            Pin(num='1',name='~DTR',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='~RTS',do_erc=True),
            Pin(num='3',name='VCCIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='RXD',do_erc=True),
            Pin(num='5',name='~RI',do_erc=True),
            Pin(num='6',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='7',name='~DSR',func=Pin.OUTPUT,do_erc=True),
            Pin(num='8',name='~DCD',do_erc=True),
            Pin(num='9',name='~CTS',do_erc=True),
            Pin(num='10',name='CBUS2',func=Pin.BIDIR,do_erc=True),
            Pin(num='20',name='TXD',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='USBDP',func=Pin.BIDIR,do_erc=True),
            Pin(num='12',name='USBDM',func=Pin.BIDIR,do_erc=True),
            Pin(num='13',name='3V3OUT',func=Pin.PWROUT,do_erc=True),
            Pin(num='14',name='~RESET',do_erc=True),
            Pin(num='15',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='17',name='CBUS1',func=Pin.BIDIR,do_erc=True),
            Pin(num='18',name='CBUS0',func=Pin.BIDIR,do_erc=True),
            Pin(num='19',name='CBUS3',func=Pin.BIDIR,do_erc=True)]),
        Part(name='FT232BM',dest=TEMPLATE,tool=SKIDL,keywords='USB Single UART FIFO',description='Hi Speed Single Channel USB UART/FIFO, LQFP-32',ref_prefix='U',num_units=1,fplist=['LQFP*'],do_erc=True,pins=[
            Pin(num='1',name='EESK',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='EEDATA',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='~RESET',do_erc=True),
            Pin(num='5',name='~RSTOUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='6',name='3V3OUT',func=Pin.PWROUT,do_erc=True),
            Pin(num='7',name='USBDP',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='USBDM',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='10',name='~SLEEP',func=Pin.OUTPUT,do_erc=True),
            Pin(num='20',name='D5/~DSR',func=Pin.BIDIR,do_erc=True),
            Pin(num='30',name='AVCC',do_erc=True),
            Pin(num='11',name='~RXLED',do_erc=True),
            Pin(num='21',name='D4/~DTR',func=Pin.BIDIR,do_erc=True),
            Pin(num='31',name='TEST',do_erc=True),
            Pin(num='12',name='~TXLED',func=Pin.OUTPUT,do_erc=True),
            Pin(num='22',name='D3/~CTS',func=Pin.BIDIR,do_erc=True),
            Pin(num='32',name='EECS',func=Pin.BIDIR,do_erc=True),
            Pin(num='13',name='VCCIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='23',name='D2/~RST',func=Pin.BIDIR,do_erc=True),
            Pin(num='14',name='PWRCTL',func=Pin.BIDIR,do_erc=True),
            Pin(num='24',name='D1/RXD',func=Pin.BIDIR,do_erc=True),
            Pin(num='15',name='~PWREN',do_erc=True),
            Pin(num='25',name='D0/TXD',func=Pin.BIDIR,do_erc=True),
            Pin(num='16',name='TXDEN',do_erc=True),
            Pin(num='26',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='17',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='27',name='XTIN',do_erc=True),
            Pin(num='18',name='D7/~RI',func=Pin.BIDIR,do_erc=True),
            Pin(num='28',name='XTOUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='19',name='D6/~DCD',func=Pin.BIDIR,do_erc=True),
            Pin(num='29',name='AGND',func=Pin.PWRIN,do_erc=True)]),
        Part(name='FT232H',dest=TEMPLATE,tool=SKIDL,keywords='USB Single UART FIFO',description='FT232Hx, Hi Speed Single Channel USB UART/FIFO, LQFP/QFN-48',ref_prefix='U',num_units=1,fplist=['LQFP*', 'QFN*'],do_erc=True,pins=[
            Pin(num='1',name='XCSI',do_erc=True),
            Pin(num='2',name='XCSO',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='VPHY',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='AGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='5',name='REF',do_erc=True),
            Pin(num='6',name='DM',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='DP',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VPLL',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='AGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='10',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='20',name='ADBUS7',func=Pin.BIDIR,do_erc=True),
            Pin(num='30',name='ACBUS6',func=Pin.BIDIR,do_erc=True),
            Pin(num='40',name='VREGIN',func=Pin.PWRIN,do_erc=True),
            Pin(num='11',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='21',name='ACBUS0',func=Pin.BIDIR,do_erc=True),
            Pin(num='31',name='ACBUS7',func=Pin.BIDIR,do_erc=True),
            Pin(num='41',name='AGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='12',name='VCCIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='22',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='32',name='ACBUS8',func=Pin.BIDIR,do_erc=True),
            Pin(num='42',name='TEST',do_erc=True),
            Pin(num='13',name='ADBUS0',func=Pin.BIDIR,do_erc=True),
            Pin(num='23',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='33',name='ACBUS9',func=Pin.BIDIR,do_erc=True),
            Pin(num='43',name='EEDATA',func=Pin.BIDIR,do_erc=True),
            Pin(num='14',name='ADBUS1',func=Pin.BIDIR,do_erc=True),
            Pin(num='24',name='VCCIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='34',name='~RESET',do_erc=True),
            Pin(num='44',name='EECLK',do_erc=True),
            Pin(num='15',name='ADBUS2',func=Pin.BIDIR,do_erc=True),
            Pin(num='25',name='ACBUS1',func=Pin.BIDIR,do_erc=True),
            Pin(num='35',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='45',name='EECS',do_erc=True),
            Pin(num='16',name='ADBUS3',func=Pin.BIDIR,do_erc=True),
            Pin(num='26',name='ACBUS2',func=Pin.BIDIR,do_erc=True),
            Pin(num='36',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='46',name='VCCIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='17',name='ADBUS4',func=Pin.BIDIR,do_erc=True),
            Pin(num='27',name='ACBUS3',func=Pin.BIDIR,do_erc=True),
            Pin(num='37',name='VCCA',func=Pin.PWROUT,do_erc=True),
            Pin(num='47',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='18',name='ADBUS5',func=Pin.BIDIR,do_erc=True),
            Pin(num='28',name='ACBUS4',func=Pin.BIDIR,do_erc=True),
            Pin(num='38',name='VCCCORE',func=Pin.PWROUT,do_erc=True),
            Pin(num='48',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='19',name='ADBUS6',func=Pin.BIDIR,do_erc=True),
            Pin(num='29',name='ACBUS5',func=Pin.BIDIR,do_erc=True),
            Pin(num='39',name='VCCD',func=Pin.PWRIN,do_erc=True)]),
        Part(name='FT232RL',dest=TEMPLATE,tool=SKIDL,keywords='USB Serial',description='USB to Serial Interface, SSOP-28',ref_prefix='U',num_units=1,fplist=['SSOP*'],do_erc=True,pins=[
            Pin(num='1',name='TXD',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='DTR',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='RTS',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='VCCIO',do_erc=True),
            Pin(num='5',name='RXD',do_erc=True),
            Pin(num='6',name='RI',do_erc=True),
            Pin(num='7',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='DCR',do_erc=True),
            Pin(num='10',name='DCD',do_erc=True),
            Pin(num='20',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='11',name='CTS',do_erc=True),
            Pin(num='21',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='12',name='CBUS4',func=Pin.BIDIR,do_erc=True),
            Pin(num='22',name='CBUS1',func=Pin.OUTPUT,do_erc=True),
            Pin(num='13',name='CBUS2',func=Pin.BIDIR,do_erc=True),
            Pin(num='23',name='CBUS0',func=Pin.BIDIR,do_erc=True),
            Pin(num='14',name='CBUS3',func=Pin.BIDIR,do_erc=True),
            Pin(num='15',name='USBD+',func=Pin.BIDIR,do_erc=True),
            Pin(num='25',name='AGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='USBD-',func=Pin.BIDIR,do_erc=True),
            Pin(num='26',name='TEST',func=Pin.PWRIN,do_erc=True),
            Pin(num='17',name='3V3OUT',do_erc=True),
            Pin(num='27',name='OSCI',do_erc=True),
            Pin(num='18',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='28',name='OSCO',func=Pin.OUTPUT,do_erc=True),
            Pin(num='19',name='~RESET~',do_erc=True)]),
        Part(name='FT240XQ',dest=TEMPLATE,tool=SKIDL,keywords='FTDI USB FIFO interface converter',description='Full Speed USB to 8-Bit FIFO, QFN-24',ref_prefix='U',num_units=1,fplist=['QFN*'],do_erc=True,pins=[
            Pin(num='1',name='DATA1',func=Pin.BIDIR,do_erc=True),
            Pin(num='2',name='DATA7',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='DATA5',func=Pin.BIDIR,do_erc=True),
            Pin(num='5',name='DATA6',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='DATA3',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='~SI/WU',do_erc=True),
            Pin(num='8',name='~RD',do_erc=True),
            Pin(num='9',name='~WR',do_erc=True),
            Pin(num='10',name='USBDP',func=Pin.BIDIR,do_erc=True),
            Pin(num='20',name='CBUS5',func=Pin.BIDIR,do_erc=True),
            Pin(num='11',name='USBDM',func=Pin.BIDIR,do_erc=True),
            Pin(num='21',name='DATA0',func=Pin.BIDIR,do_erc=True),
            Pin(num='12',name='3V3OUT',func=Pin.PWROUT,do_erc=True),
            Pin(num='22',name='DATA4',func=Pin.BIDIR,do_erc=True),
            Pin(num='13',name='~RESET',do_erc=True),
            Pin(num='23',name='DATA2',func=Pin.BIDIR,do_erc=True),
            Pin(num='14',name='VCCIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='24',name='VCCIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='15',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='25',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='17',name='~TXE',func=Pin.OUTPUT,do_erc=True),
            Pin(num='18',name='~RXF',func=Pin.OUTPUT,do_erc=True),
            Pin(num='19',name='CBUS6',func=Pin.BIDIR,do_erc=True)]),
        Part(name='FT240XS',dest=TEMPLATE,tool=SKIDL,keywords='FTDI USB FIFO interface converter',description='Full Speed USB to 8-Bit FIFO, SSOP-24',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='DATA4',func=Pin.BIDIR,do_erc=True),
            Pin(num='2',name='DATA2',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='VCCIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='DATA1',func=Pin.BIDIR,do_erc=True),
            Pin(num='5',name='DATA7',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='7',name='DATA5',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='DATA6',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='DATA3',func=Pin.BIDIR,do_erc=True),
            Pin(num='10',name='~SI/WU',do_erc=True),
            Pin(num='20',name='~TXE',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='~RD',do_erc=True),
            Pin(num='21',name='~RXF',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='~WR',do_erc=True),
            Pin(num='22',name='CBUS6',func=Pin.BIDIR,do_erc=True),
            Pin(num='13',name='USBDP',func=Pin.BIDIR,do_erc=True),
            Pin(num='23',name='CBUS5',func=Pin.BIDIR,do_erc=True),
            Pin(num='14',name='USBDM',func=Pin.BIDIR,do_erc=True),
            Pin(num='24',name='DATA0',func=Pin.BIDIR,do_erc=True),
            Pin(num='15',name='3V3OUT',func=Pin.PWROUT,do_erc=True),
            Pin(num='16',name='~RESET',do_erc=True),
            Pin(num='17',name='VCCIO',func=Pin.BIDIR,do_erc=True),
            Pin(num='18',name='VCC',func=Pin.BIDIR,do_erc=True),
            Pin(num='19',name='GND',func=Pin.BIDIR,do_erc=True)]),
        Part(name='FT245BM',dest=TEMPLATE,tool=SKIDL,keywords='FTDI USB FIFO Interface Converter',description='Full Speed USB to 8-Bit FIFO, LQFP-32',ref_prefix='U',num_units=1,fplist=['LQFP*'],do_erc=True,pins=[
            Pin(num='1',name='EESK',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='EEDATA',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='~RESET',do_erc=True),
            Pin(num='5',name='~RSTOUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='6',name='3V3OUT',func=Pin.PWROUT,do_erc=True),
            Pin(num='7',name='USBDP',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='USBDM',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='10',name='~PWREN',func=Pin.OUTPUT,do_erc=True),
            Pin(num='20',name='D5',func=Pin.BIDIR,do_erc=True),
            Pin(num='30',name='AVCC',do_erc=True),
            Pin(num='11',name='SI/WU',do_erc=True),
            Pin(num='21',name='D4',func=Pin.BIDIR,do_erc=True),
            Pin(num='31',name='TEST',do_erc=True),
            Pin(num='12',name='~RXF',func=Pin.OUTPUT,do_erc=True),
            Pin(num='22',name='D3',func=Pin.BIDIR,do_erc=True),
            Pin(num='32',name='EECS',func=Pin.BIDIR,do_erc=True),
            Pin(num='13',name='VCCIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='23',name='D2',func=Pin.BIDIR,do_erc=True),
            Pin(num='14',name='~TXE',func=Pin.OUTPUT,do_erc=True),
            Pin(num='24',name='D1',func=Pin.BIDIR,do_erc=True),
            Pin(num='15',name='WR',do_erc=True),
            Pin(num='25',name='D0',func=Pin.BIDIR,do_erc=True),
            Pin(num='16',name='~RD',do_erc=True),
            Pin(num='26',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='17',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='27',name='XTIN',do_erc=True),
            Pin(num='18',name='D7',func=Pin.BIDIR,do_erc=True),
            Pin(num='28',name='XTOUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='19',name='D6',func=Pin.BIDIR,do_erc=True),
            Pin(num='29',name='GND',func=Pin.PWRIN,do_erc=True)]),
        Part(name='FT4232H',dest=TEMPLATE,tool=SKIDL,keywords='USB Quad UART FIFO',description='Hi Speed Quad Channel USB UART/FIFO, LQFP/QFN-64',ref_prefix='U',num_units=1,fplist=['LQFP*', 'QFN*'],do_erc=True,pins=[
            Pin(num='1',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='OSCI',do_erc=True),
            Pin(num='3',name='OSCO',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='VPHY',func=Pin.PWRIN,do_erc=True),
            Pin(num='5',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='6',name='REF',do_erc=True),
            Pin(num='7',name='DM',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='DP',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='VPLL',func=Pin.PWRIN,do_erc=True),
            Pin(num='10',name='AGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='20',name='VCCIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='30',name='BDBUS4',func=Pin.BIDIR,do_erc=True),
            Pin(num='40',name='CDBUS2',func=Pin.BIDIR,do_erc=True),
            Pin(num='50',name='VREGIN',func=Pin.PWRIN,do_erc=True),
            Pin(num='60',name='~PWREN',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='21',name='ADBUS4',func=Pin.BIDIR,do_erc=True),
            Pin(num='31',name='VCCIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='41',name='CDBUS3',func=Pin.BIDIR,do_erc=True),
            Pin(num='51',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='61',name='EEDATA',func=Pin.BIDIR,do_erc=True),
            Pin(num='12',name='VCORE',func=Pin.PWRIN,do_erc=True),
            Pin(num='22',name='ADBUS5',func=Pin.BIDIR,do_erc=True),
            Pin(num='32',name='BDBUS5',func=Pin.BIDIR,do_erc=True),
            Pin(num='42',name='VCCIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='52',name='DDBUS1',func=Pin.BIDIR,do_erc=True),
            Pin(num='62',name='EECLK',func=Pin.OUTPUT,do_erc=True),
            Pin(num='13',name='TEST',do_erc=True),
            Pin(num='23',name='ADBUS6',func=Pin.BIDIR,do_erc=True),
            Pin(num='33',name='BDBUS6',func=Pin.BIDIR,do_erc=True),
            Pin(num='43',name='CDBUS4',func=Pin.BIDIR,do_erc=True),
            Pin(num='53',name='DDBUS2',func=Pin.BIDIR,do_erc=True),
            Pin(num='63',name='EECS',func=Pin.OUTPUT,do_erc=True),
            Pin(num='14',name='~RESET',do_erc=True),
            Pin(num='24',name='ADBUS7',func=Pin.BIDIR,do_erc=True),
            Pin(num='34',name='BDBUS7',func=Pin.BIDIR,do_erc=True),
            Pin(num='44',name='CDBUS5',func=Pin.BIDIR,do_erc=True),
            Pin(num='54',name='DDBUS3',func=Pin.BIDIR,do_erc=True),
            Pin(num='64',name='VCORE',func=Pin.PWRIN,do_erc=True),
            Pin(num='15',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='25',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='35',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='45',name='CDBUS6',func=Pin.BIDIR,do_erc=True),
            Pin(num='55',name='DDBUS4',func=Pin.BIDIR,do_erc=True),
            Pin(num='16',name='ADBUS0',func=Pin.BIDIR,do_erc=True),
            Pin(num='26',name='BDBUS0',func=Pin.BIDIR,do_erc=True),
            Pin(num='36',name='~SUSPEND',func=Pin.OUTPUT,do_erc=True),
            Pin(num='46',name='CDBUS7',func=Pin.BIDIR,do_erc=True),
            Pin(num='56',name='VCCIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='17',name='ADBUS1',func=Pin.BIDIR,do_erc=True),
            Pin(num='27',name='BDBUS1',func=Pin.BIDIR,do_erc=True),
            Pin(num='37',name='VCORE',func=Pin.PWRIN,do_erc=True),
            Pin(num='47',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='57',name='DDBUS5',func=Pin.BIDIR,do_erc=True),
            Pin(num='18',name='ADBUS2',func=Pin.BIDIR,do_erc=True),
            Pin(num='28',name='BDBUS2',func=Pin.BIDIR,do_erc=True),
            Pin(num='38',name='CDBUS0',func=Pin.BIDIR,do_erc=True),
            Pin(num='48',name='DDBUS0',func=Pin.BIDIR,do_erc=True),
            Pin(num='58',name='DDBUS6',func=Pin.BIDIR,do_erc=True),
            Pin(num='19',name='ADBUS3',func=Pin.BIDIR,do_erc=True),
            Pin(num='29',name='BDBUS3',func=Pin.BIDIR,do_erc=True),
            Pin(num='39',name='CDBUS1',func=Pin.BIDIR,do_erc=True),
            Pin(num='49',name='VREGOUT',func=Pin.PWROUT,do_erc=True),
            Pin(num='59',name='DDBUS7',func=Pin.BIDIR,do_erc=True)])])